#include "StdAfx.h"
#include "Set.h"

/////////////////////////////////////////////////////////////////////////
//  START - Set Support, Convert Later to a Set Class with overloaded
//  operators for: +, -, *, /
CSet::CSet(){
	Bytes = NULL;
	NumberOfElements = 0;
	NumberOFBytes = 0;
}

CSet::CSet(long NumOfElem){		//  Constructor

	NumberOfElements = NumOfElem;
	NumberOFBytes = NumberOfElements / 8;
	if(NumberOfElements % 8)
	   NumberOFBytes++;

	if(NumberOFBytes){
	   Bytes = new char[NumberOFBytes];
	   memset(Bytes, 0, NumberOFBytes);
	   }
	   else Bytes = NULL;
}								//  End of Constructor

CSet::CSet(const CSet &other){		//  Copy Construcotr
	NumberOfElements = other.NumberOfElements;
	NumberOFBytes = other.NumberOFBytes;
	Bytes = new char[NumberOFBytes];
	memcpy(Bytes, other.Bytes, NumberOFBytes);
}								//  End of Copy Construcotr

CSet &CSet::operator=(const CSet &other){		//  Operator=
    if(this == &other)
	   return *this;

	if(!other.Bytes){
	   delete [] Bytes;
	   Bytes = NULL;
	   NumberOfElements = 0;
	   NumberOFBytes = 0;
	   return *this;
	   }

	if(NumberOFBytes != other.NumberOFBytes){
	   delete [] Bytes;
	   NumberOFBytes = other.NumberOFBytes;
	   Bytes = new char[NumberOFBytes];
	}

	NumberOfElements = other.NumberOfElements;
	memcpy(Bytes, other.Bytes, NumberOFBytes);

	return *this;
}								//  End of Operator=


CSet::~CSet(){					//  Destructor
	if(Bytes) delete [] Bytes;
}

void CSet::AddToSet(long Index){
	if(!Bytes) return;

	char ORMask = 1;
	long SIndex = Index / 8;

	ORMask <<= Index % 8;
	if(SIndex <= NumberOFBytes)
	   Bytes[SIndex] |= ORMask;
}

void CSet::AddPathToSet(const CWay &Way){
	if(!Bytes) return;

	char ORMask;
	long SIndex;

	for(int i = 0; i <= Way.GetNextCity(); i++){
		ORMask = 1;
		SIndex = Way.GetCity(i) / 8;
		ORMask <<= Way.GetCity(i) % 8;
		if(SIndex <= NumberOFBytes)
		   Bytes[SIndex] |= ORMask;
	}
}

void CSet::RemoveFromSet(long Index){
	if(!Bytes) return;

	char XORMask = 1;
	long SIndex = Index / 8;

	if(SIndex > NumberOFBytes) return;

	if(Bytes[SIndex] & XORMask){
	   XORMask <<= Index % 8;
	   Bytes[SIndex] ^= XORMask;
	   }
}

int CSet::operator==(const CSet &other){
	if(!Bytes) return 1;
	int Result = TRUE;

	for(int i = 0; i < NumberOFBytes; i++){
		Result = Bytes[i] - other.Bytes[i];
		if(Result) return 0;
	}

	return TRUE;
}							//  End - operator== Function

int CSet::operator!=(const CSet &other){
	if(!Bytes) return TRUE;
	int Result = FALSE;

	for(int i = 0; i < NumberOFBytes; i++){
		Result = Bytes[i] - other.Bytes[i];
		if(Result) return Result;
	}

	return FALSE;
}							//  End - operator!= Function

int CSet::InSet(long Value) const{
	if(!Bytes) return FALSE;

	char ANDMask = 1;
	long SIndex = Value / 8;

	if(SIndex > NumberOFBytes) return 0;
	ANDMask <<= Value % 8;

	return (int) Bytes[SIndex] & ANDMask;
}

//  END - Set Support
/////////////////////////////////////////////////////////////////////////