/////////////////////////////////////////////////////////////////////////////
// CStack class
// All Stack function are defined in Stack.h as  inline functions
#include "stdafx.h"
#include "Stack.h"
#include "Way.h"
#include "Set.h"
#include "TAgentDoc.h"

int compare(const void *Way1, const void *Way2);

CStack::CStack()
	: CPtrArray(){					//  Calling Base class Constructor
	Top = 0;
	SetSize(MAX_STACK_SIZE);		//  Initiate Size too MAX_STACK_SIZE
}

void CStack::Push(CWay *newElement){
	int Size =  GetSize();
	if(Top == Size)
	   SetSize(Size + 10, 10);
	   
	SetAt(Top, (void *) newElement);
	Top++;
	TRACE("%ld Pushed CWay %p, NextCity = %ld, Sum = %ld\n", Top-1, newElement, newElement->GetNextCity(), newElement->GetSum());
}



CWay *CStack::Pop(){
	if(Top){
	   Top--;
	   CWay *Element = (CWay *) GetAt(Top);
	TRACE("%ld Poped CWay %p, NextCity = %ld, Sum = %ld\n", Top, Element, Element->GetNextCity(), Element->GetSum());
	   return Element;
	}

	TRACE("Poped a NULL pointer");
	return NULL;
}

void CStack::DisposeStack(){
	CWay *pWay;
	while(IsNOTEmpty()){			//  Realse Memeory of Stack
	  pWay = Pop();
	  delete pWay;					//  Release Memeory   (DISPOSE(pWay))
	}
}

void *CStack::SaveSearchInfo(UINT SearchNumber, char *MapName, CWay *Request, CWay *BestSolution){

	long Temp;
	ULONG MemSize = sizeof(SREARCHFILEHEADER)							//  Space for Search File Header
		    + sizeof(long)*Request->GetNumberOfCities()					//  Space for Request Path
		    + sizeof(long)*Request->GetNumberOfCities()					//  Space for BestSolution Path
		    + sizeof(long)*(Request->GetNumberOfCities() + 2)*Top;	    //  Space for: ((Path + NextCity + Sum)) * Top
	void *Buffer = (void *) new char[MemSize + sizeof(long)];			//  Allocate memory space to save MemSize as well
	char *p;
	CWay *Record;
	SREARCHFILEHEADER Header;
	TRACE("SaveSearchInfo: MemSize = %ld\n", MemSize);

	strcpy(Header.MapName, MapName); 		//  START - Buiding File Header
	Header.FirstCity = Request->GetFirstCity();
	Header.LastCity = Request->GetLastCity();
	Header.NumberOfCities = Request->GetNumberOfCities();
	Header.BestSolSum = BestSolution->GetSum();
	Header.StackTop = Top;					//  END - Buiding File Header

	p = (char *) Buffer;
	memcpy(p, (char *) &MemSize, sizeof(long));				//  Saving MemSize first
	p += sizeof(long);

	memcpy(p, (char *) &Header, sizeof(SREARCHFILEHEADER));	//  Saving File Header
	p += sizeof(SREARCHFILEHEADER);

	memcpy(p, (char *) Request->GetPath(), sizeof(long)*Request->GetNumberOfCities());
	p += sizeof(long)*Request->GetNumberOfCities();  //  Save Request Path

	memcpy(p, (char *) BestSolution->GetPath(), sizeof(long)*BestSolution->GetNumberOfCities());
	p += sizeof(long)*Request->GetNumberOfCities();		  //  Save BestSolution Path

	for(long i = 0; i < Top; i++){						  //  Saving Stack content
		Record = (CWay *) GetAt(i);
		TRACE("Saved: %ld Record: NextCity = %ld, Sum = %ld\n", i, Record->GetNextCity(), Record->GetSum());
		memcpy(p, (char *) Record->GetPath(), sizeof(long)*Request->GetNumberOfCities());
		p += sizeof(long)*Request->GetNumberOfCities();  //  Save Path
		Temp = Record->GetNextCity();					 //  Save NextCity
		memcpy(p, (char *) &Temp, sizeof(long));
		p += sizeof(long);
		Temp = Record->GetSum();						 //  Save Sum
		memcpy(p, (char *) &Temp, sizeof(long));
		p += sizeof(long);
	}					//  End of for(long i...

	return Buffer;		//  Add the Time information to Saved File (No Space in the Buffer for it
}						//  End of SaveSearchInfo Function

void CStack::LoadStack(void *FromFile, const CWay &SearchDef, CWay *Solution){
	char *p = (char *) FromFile 
			  + sizeof(SREARCHFILEHEADER)
			  + (SearchDef.GetNumberOfCities())*(sizeof(long));

	SREARCHFILEHEADER *Header = (SREARCHFILEHEADER *) FromFile;
	Solution->SetSum(Header->BestSolSum);							  //  Set Sum of Solution
	memcpy((char *) Solution->GetPath(), p, sizeof(long)*Solution->GetNumberOfCities());
	p += sizeof(long)*SearchDef.GetNumberOfCities();				  //  Get Best Solution Path
	Solution->SetNextCity(Header->NumberOfCities-1);					  //  Set Solution NextCity

	CWay *Record;
	long *Temp;

	for(long i = 0; i < Header->StackTop; i++){						  //  Load Stack content
		Record = new CWay(SearchDef);
		memcpy((char *) Record->GetPath(), p, sizeof(long)*SearchDef.GetNumberOfCities());
		p += sizeof(long)*SearchDef.GetNumberOfCities();  //  Save Path
		Temp = (long *) p;
		Record->SetNextCity(*Temp);				   //  Get NextCity
		p += sizeof(long);
		Temp = (long *) p;
		Record->SetSum(*Temp);					   //  Get Sum
		p += sizeof(long);
		Push(Record);
	}

	p = (char *) FromFile;
	TRACE("LoadStack: Deleted FromFile Momory allocation\n");
	delete [] p;
}						//  End of LoadStack Function

/////////////////////////////////////////////////////////////////////////
//  START - Branch and Bound routines
BOOL CStack::PushSons(const CWay *Way, const CSet &Request, CTAgentDoc *pDoc){
	long Low = Top;
	long Index;
	CSet Cities(pDoc->MapHeader.NumberOfCities);
	CWay *pWay;

	Cities.AddToSet(Way->GetFirstCity());
	Cities.AddToSet(Way->GetLastCity());

	if(Way->GetNextCity())
	   Cities.AddPathToSet(*Way);

	Index = 0;
	while(Cities != Request){
	  if((Request.InSet(Index)) && (!(Cities.InSet(Index)))){
		 Cities.AddToSet(Index);
		 pWay = new CWay(*Way);
		 pWay->AddToSum(pDoc->GetDistance(Way->GetCity(Way->GetNextCity()), Index));
		 pWay->AddCityToPath(Index);
		 Push(pWay);
	  }

	  Index++;
	  if(Index == Way->GetCity(Way->GetNextCity()))
		 Index++;
	}

	if(Low != Top){
	   TRACE("Sorting %ld <-> %ld,  %ld Elements \n", Low, Top-1, Top - Low);
	   Sort(Low, Top - 1);
#ifdef _DEBUG
	   for(long k = Low; k < Top; k++)	   //  Remove After Debug
		   TRACE("At place %ld CWay Element %p NextCity = %ld, Sum = %ld\n", k, GetAt(k), ((CWay *) GetAt(k))->GetNextCity(),((CWay *) GetAt(k))->GetSum());
#endif
//	   qsort((void *) &ElementAt(Low), Top - Low, sizeof(CWay *), compare);
	   return FALSE;
	}
	else {
#ifdef _DEBUG
		   for(long k = 0; k < Way->GetNumberOfCities(); k++)	   //  Remove After Debug
			   TRACE("Solution: NextCity = %ld City %ld = %ld\n",Way->GetNextCity(), k, Way->GetCity(k));
#endif		
		   return TRUE;
	}
}				//  End of PushSons Function

//int compare(const void *Way1, const void *Way2){
//	CWay *pWay1 = (CWay *) Way1;
//	CWay *pWay2 = (CWay *) Way2;
//	return pWay1->GetSum() - pWay2->GetSum();
//}

void CStack::Sort(long Low, long High){		//  Sort Pointers by Sum
	CWay *Hold;
	long I, J;
	ULONG K;

	I = Low;
	J = High;
	K =  ((CWay *) GetAt((Low + High) / 2))->GetSum();

	do{
		while((((CWay *) GetAt(I))->GetSum() > (K)) && (I < High)) I++;
		while((((CWay *) GetAt(J))->GetSum() < (K)) && (J > Low)) J--;

		if(I <= J){
		   if(((CWay *) GetAt(I))->GetSum() != ((CWay *) GetAt(J))->GetSum()){
		      Hold = (CWay *) GetAt(I);
			  SetAt(I, GetAt(J));
			  SetAt(J, Hold);
		   }
		   I++;
		   J--;
		}
	}while(I <= J);

	if(Low < J) Sort(Low, J);
	if(I < High) Sort(I, High);
}					//  End of Sort Function

//  END - Branch and Bound routines
/////////////////////////////////////////////////////////////////////////